@@ -12,15 +12,16 @@ class PreNormResidual(nn.Module):
         return self.fn(self.norm(x)) + x
 
 def FeedForward(dim, expansion_factor = 4, dropout = 0., dense = nn.Linear):
+    inner_dim = int(dim * expansion_factor)
     return nn.Sequential(
-        dense(dim, dim * expansion_factor),
+        dense(dim, inner_dim),
         nn.GELU(),
         nn.Dropout(dropout),
-        dense(dim * expansion_factor, dim),
+        dense(inner_dim, dim),
         nn.Dropout(dropout)
     )
 
-def MLPMixer(*, image_size, channels, patch_size, dim, depth, num_classes, expansion_factor = 4, dropout = 0.):
+def MLPMixer(*, image_size, channels, patch_size, dim, depth, num_classes, expansion_factor = 4, expansion_factor_token = 0.5, dropout = 0.):
     assert (image_size % patch_size) == 0, 'image must be divisible by patch size'
     num_patches = (image_size // patch_size) ** 2
     chan_first, chan_last = partial(nn.Conv1d, kernel_size = 1), nn.Linear
@@ -30,7 +31,7 @@ def MLPMixer(*, image_size, channels, patch_size, dim, depth, num_classes, expan
         nn.Linear((patch_size ** 2) * channels, dim),
         *[nn.Sequential(
             PreNormResidual(dim, FeedForward(num_patches, expansion_factor, dropout, chan_first)),
-            PreNormResidual(dim, FeedForward(dim, expansion_factor, dropout, chan_last))
+            PreNormResidual(dim, FeedForward(dim, expansion_factor_token, dropout, chan_last))
         ) for _ in range(depth)],
         nn.LayerNorm(dim),
         Reduce('b n c -> b c', 'mean'),
